%% TFM - Lucas Onrubia Fontangordo
%% Limpieza del espacio de trabajo

close all
clear all
clc

%% Carga de los datos

load(uigetfile('*.mat'));
% Elegimos un vector de frecuencias, será el mismo para todas las medidas
f = M1P1A(:,1);
% Quitamos la primera columna de todas las medidas, ya que es la frecuencia
% y es la misma para todas (ya tenemos guardado el vector f para eso)
M1P1A(:,1) = [];
M1P2A(:,1) = [];
M1P3A(:,1) = [];
M1P1F(:,1) = [];
M1P2F(:,1) = [];
M1P3F(:,1) = [];
M2P1A(:,1) = [];
M2P2A(:,1) = [];
M2P3A(:,1) = [];
M2P1F(:,1) = [];
M2P2F(:,1) = [];
M2P3F(:,1) = [];
M3P1A(:,1) = [];
M3P2A(:,1) = [];
M3P3A(:,1) = [];
M3P1F(:,1) = [];
M3P2F(:,1) = [];
M3P3F(:,1) = [];

%% 'Pequeñas' operaciones con los datos recogidos

% % Descomentar si se quiere hacer el smoothing antes de promediar los datos,
% % sino se hará después de hacer el promedio. los resultados varían poco se
% % haga el smoothing antes o después.
% smoothing = input('¿Deseas suavizar los datos originales? (S/N) {Recomendable suavizar}: ','s');
% if(smoothing == 'S'||smoothing == 's')
%     metodo = 'loess';
%     M1P1A = smoothdata(M1P1A,metodo);M2P1A = smoothdata(M2P1A,metodo); M3P1A = smoothdata(M3P1A,metodo);
%     M1P2A = smoothdata(M1P2A,metodo);M2P2A = smoothdata(M2P2A,metodo); M3P2A = smoothdata(M3P2A,metodo);
%     M1P3A = smoothdata(M1P3A,metodo);M2P3A = smoothdata(M2P3A,metodo); M3P3A = smoothdata(M3P3A,metodo);
%     M1P1F = smoothdata(M1P1F,metodo);M2P1F = smoothdata(M2P1F,metodo); M3P1F = smoothdata(M3P1F,metodo);
%     M1P2F = smoothdata(M1P2F,metodo);M2P2F = smoothdata(M2P2F,metodo); M3P2F = smoothdata(M3P2F,metodo);
%     M1P3F = smoothdata(M1P3F,metodo);M2P3F = smoothdata(M2P3F,metodo); M3P3F = smoothdata(M3P3F,metodo);
% else
% end

% Promediado de los resultados para consolidar una sola muestra a partir de
% las 3 que hemos medido
for i=1:length(f)
    % Aceleración, posiciones 1 a 3
    P1A(i,:) = (M1P1A(i)+M2P1A(i)+M3P1A(i))./3;
    P2A(i,:) = (M1P2A(i)+M2P2A(i)+M3P2A(i))./3;
    P3A(i,:) = (M1P3A(i)+M2P3A(i)+M3P3A(i))./3;
    % Fuerza, posiciones 1 a 3
    P1F(i,:) = (M1P1F(i)+M2P1F(i)+M3P1F(i))./3;
    P2F(i,:) = (M1P2F(i)+M2P2F(i)+M3P2F(i))./3;
    P3F(i,:) = (M1P3F(i)+M2P3F(i)+M3P3F(i))./3;
end


% % Smoothing de los datos recogidos, útil para quitar la gran cantidad de
% % ruido que hay en los datos y para limpiarlos. útil también en el cálculo
% % de la función de transferencia.
smoothing = input('¿Deseas suavizar los datos originales? (S/N) {Recomendable suavizar}: ','s');
if(smoothing == 'S'||smoothing == 's')
    metodo = 'loess';
    P1A = smoothdata(P1A,metodo);P2A = smoothdata(P2A,metodo); P3A = smoothdata(P3A,metodo);
    P1F = smoothdata(P1F,metodo);P2F = smoothdata(P2F,metodo); P3F = smoothdata(P3F,metodo);
else
end

% Representación de las medidas de la muestra:
figure(1);
subplot(2,3,1)
plot(f,P1A)
xlabel('Frecuencia (Hz)')
ylabel('Aceleración (m/s^2)')
grid
title('Posición 1')
subplot(2,3,2)
plot(f,P2A)
xlabel('Frecuencia (Hz)')
ylabel('Aceleración (m/s^2)')
grid
title('Posición 2')
subplot(2,3,3)
plot(f,P3A)
xlabel('Frecuencia (Hz)')
ylabel('Aceleración (m/s^2)')
grid
title('Posición 3')

subplot(2,3,4)
plot(f,P1F)
xlabel('Frecuencia (Hz)')
ylabel('Fuerza (N)')
grid
title('Posición 1')
subplot(2,3,5)
plot(f,P2F)
xlabel('Frecuencia (Hz)')
ylabel('Fuerza (N)')
grid
title('Posición 2')
subplot(2,3,6)
plot(f,P3F)
xlabel('Frecuencia (Hz)')
ylabel('Fuerza (N)')
grid
title('Posición 3')

%% Procesado

% Para poder realizar la función de transferencia necesitamos dividir un
% desplazamiento entre una fuerza. Para obtener el desplazamiento hay que
% derivar dos veces los datos de aceleración que tenemos.

% Obtención de los datos de desplazamiento a partir de los de aceleración.
% Usamos integración numérica (cumtrapz).
% Según Brüel la frecuencia de muestreo es 2,56 veces la frecuencia máxima
% medida.
Fs = 2.56*f(end);
% Con esto obtenemos el tiempo de muestreo, que es un parámetro necesario
% para poder calcular la integral numérica
Ts = 1/Fs;
% Ahora tenemos que realizar la integración numérica dos veces para los
% datos de aceleración y así sacar el desplazamiento:
% Primera derivada, Velocidad:
VelP1 = cumtrapz(Ts,P1A);
VelP2 = cumtrapz(Ts,P2A);
VelP3 = cumtrapz(Ts,P3A);
% Segunda derivada, Desplazamiento:
DesplP1 = cumtrapz(Ts,VelP1);
DesplP2 = cumtrapz(Ts,VelP2);
DesplP3 = cumtrapz(Ts,VelP3);

% Calculamos las funciones de transferencia en los 3 puntos medidos, según
% Brüel es la compliancia del sistema (o FRF):
HP1 = DesplP1./P1F;
HP2 = DesplP2./P2F;
HP3 = DesplP3./P3F;

figure(2);
subplot(2,3,1)
plot(f,VelP1);
xlabel('Frecuencia (hz)')
ylabel('Velocidad (m/s)')
title('Posición 1')
grid
subplot(2,3,2)
plot(f,VelP2);
xlabel('Frecuencia (hz)')
ylabel('Velocidad (m/s)')
title('Posición 2')
grid
subplot(2,3,3)
plot(f,VelP1);
xlabel('Frecuencia (hz)')
ylabel('Velocidad (m/s)')
title('Posición 3')
grid
subplot(2,3,4)
plot(f,DesplP1);
xlabel('Frecuencia (hz)')
ylabel('Desplazamiento (m)')
title('Posición 1')
grid
subplot(2,3,5)
plot(f,DesplP2);
xlabel('Frecuencia (hz)')
ylabel('Desplazamiento (m)')
title('Posición 2')
grid
subplot(2,3,6)
plot(f,DesplP3);
xlabel('Frecuencia (hz)')
ylabel('Desplazamiento (m)')
title('Posición 3')
grid

% Gráficas de la función de transferencia. Limitamos el eje x a la 'zona de
% interés', fuera de ahí la gráfica se sale de los límites
% límite de frecuencia:
lim = 1200;
index = find(f==lim);

figure(3);
subplot(3,1,1)
plot(f,HP1,'k')
xlim([0 lim])
xlabel('Frecuencia (Hz)')
ylabel('H(f)')
title('Función de transferencia Posición 1')
grid
subplot(3,1,2)
plot(f,HP2,'k')
xlim([0 lim])
xlabel('Frecuencia (Hz)')
ylabel('H(f)')
title('Función de transferencia Posición 2')
grid
subplot(3,1,3)
plot(f,HP3,'k')
xlim([0 lim])
xlabel('Frecuencia (Hz)')
ylabel('H(f)')
title('Función de transferencia Posición 3')
grid

% Frecuencias de resonancia mecánicas dentro del rango ampliado de
% análisis:
% En el primer punto de medida:
rHP1 = HP1(1:index);
[m1 i1] = max(rHP1);
f1 = f(i1); w1=2*pi*f1;
% En el segundo punto de medida:
rHP2 = HP2(1:index);
[m2 i2] = max(rHP2);
f2 = f(i2); w2=2*pi*f2;
% En el tercer punto de medida:
rHP3 = HP3(1:index);
[m3 i3] = max(rHP3);
f3 = f(i3); w3=2*pi*f3;

% La rigidez estática, k, se puede obtener despejando la frecuencia de resonancia
% mecánica. Para ello necesitamos la masa m de cada una de las muestras.
disp(' ')
prompt = 'Introduce la masa media de las muestras (en gramos): ';
m = input(prompt);
m = m*10^-3; %Pasamos de gramos a Kg
disp(' ')

% Rigidez (en N/m)
k1 = m*(w1^2);
k2 = m*(w2^2);
k3 = m*(w3^2);

LF = input('Deseas hacer el cálculo del Loss Factor mediante método visual? (S/N): ','s');
if(LF == 'S'||LF == 's')
    disp(['Puede que este método NO funcione correctamente para todas las curvas...'])
    disp(' ')
    % Cálculo del loss factor (usamos los datos de aceleración en lugar de la
    % FRF:
    % Primero encontramos los picos en frecuencia para la aceleración
    figure(4); plot(f(1:round(length(f)./2)),P1A(1:round(length(P1A)./2)));
    xlabel('Frecuencia (Hz)')
    ylabel('Aceleración')
    title('Selecciona el máximo correspondiente a la frecuencia fundamental (1-3)')
    [locs1 pks1] = ginput(1);
    figure(5); plot(f(1:round(length(f)./2)),P2A(1:round(length(P2A)./2)));
    xlabel('Frecuencia (Hz)')
    ylabel('Aceleración')
    title('Selecciona el máximo correspondiente a la frecuencia fundamental (2-3)')
    [locs2 pks2] = ginput(1);
    figure(6); plot(f(1:round(length(f)./2)),P3A(1:round(length(P3A)./2)));
    xlabel('Frecuencia (Hz)')
    ylabel('Aceleración')
    title('Selecciona el máximo correspondiente a la frecuencia fundamental (3-3)')
    [locs3 pks3] = ginput(1);
    
    close(figure(4),figure(5),figure(6))
    
    % Indexamos las frecuencias
    [d4,locs1] = min(abs(f-locs1));
    [d5,locs2] = min(abs(f-locs2));
    [d6,locs3] = min(abs(f-locs3));
    
    
    % Ahora con esos picos, vamos a calcular el 3dB Bandwidth, viendo a que
    % frecuencias la amplitud máxima cae 0.707 veces tanto a un lado como a
    % otro de la curva, con eso sacamos el loss factor:
    % Márgen:
    M = 200;
    A = round(locs1-M); B = round(locs1+M);
    C = round(locs2-M); D = round(locs2+M);
    E = round(locs3-M); F = round(locs3+M);
    
    figure(7);
    plot(f(A:B),P1A(A:B));
    hold on
    plot(f(A:B),ones(length(A:B)).*(pks1*0.707));
    xlabel('Frecuencia (Hz)')
    ylabel('Aceleración')
    xlim([f(A) f(B)])
    title('Loss Factor: Selecciona los 2 puntos de corte con la curva horizontal (1-3)')
    [f3dB1 ~] = ginput(2);
    
    figure(8);
    plot(f(C:D),P2A(C:D));
    hold on
    plot(f(C:D),ones(length(C:D)).*(pks2*0.707));
    xlabel('Frecuencia (Hz)')
    ylabel('Aceleración')
    xlim([f(C) f(D)])
    title('Loss Factor: Selecciona los 2 puntos de corte con la curva horizontal (2-3)')
    [f3dB2 ~] = ginput(2);
    
    figure(9);
    plot(f(E:F),P3A(E:F));
    hold on
    plot(f(E:F),ones(length(E:F)).*(pks3*0.707));
    xlabel('Frecuencia (Hz)')
    ylabel('Aceleración')
    xlim([f(E) f(F)])
    title('Loss Factor: Selecciona los 2 puntos de corte con la curva horizontal (3-3)')
    [f3dB3 ~] = ginput(2);
    
    close(figure(7),figure(8),figure(9))
    
    LF1 = (abs(f3dB1(1)-f3dB1(2)))./f1;
    LF2 = (abs(f3dB2(1)-f3dB2(2)))./f2;
    LF3 = (abs(f3dB3(1)-f3dB3(2)))./f3;
    disp('Loss Factor: ')
    LF = [LF1 LF2 LF3];
    oLF = isoutlier(LF);
    if(oLF(1)==1)
        disp('Atención: El Loss Factor en la posición 1 es outlier')
    else
        if(oLF(2)==1)
            disp('Atención: El Loss Factor en la posición 2 es outlier')
        else
            if(oLF(3)==1)
                disp('Atención: El Loss Factor en la posición 3 es outlier')
            end
        end
    end
    disp(['El Loss Factor en la posición 1 es: ', num2str(round(LF1,3))])
    disp(['El Loss Factor en la posición 2 es: ', num2str(round(LF2,3))])
    disp(['El Loss Factor en la posición 3 es: ', num2str(round(LF3,3))])
else
    disp(['Has elegido seguir sin calcular el Loss Factor...'])
end
disp(' ')
disp('Rigidez: ')
k = [k1 k2 k3];
ok = isoutlier(k);
if(ok(1)==1)
    disp('Atención: La rigidez en la posición 1 es outlier')
else
    if(ok(2)==1)
        disp('Atención: La rigidez en la posición 2 es outlier')
    else
        if(ok(3)==1)
            disp('Atención: La rigidez en la posición 3 es outlier')
        end
    end
end
disp(['La Rigidez Estática en la posición 1 es: ', num2str(round((k1./1000),0)), ' kN/m'])
disp(['La Rigidez Estática en la posición 2 es: ', num2str(round((k2./1000),0)), ' kN/m'])
disp(['La Rigidez Estática en la posición 3 es: ', num2str(round((k3./1000),0)), ' kN/m'])

% Cálculo de módulo de Young:
% Para el cálculo necesitamos el espesor de las muestras, en nuestro caso
% todas son idénticas, circulares y de 4mm de espesor:
e0 = 4*(10^-3); % En metros
% También conocemos el radio de la muestra circular, 4cm:
a = 4*(10^-2); % En metros
% Con los datos del peso de la muestra podemos conocer la densidad:
% Calculamos el volumen de la muestra:
v = pi*(a^2)*e0; % En m3
% Deducimos la densidad:
rho = m/v; % En Kg/m3

% Según el libro de Recuero y la norma ASTM E756, y considerando que el
% coeficiente de Poisson para un material rígido es 0.1 o menos; podemos
% considerar el módulo de Young como (consideramos la fórmula donde hay una
% placa apoyada en su contorno circular):
% En teoría habría que elegir un número de modo n, elegiremos n=1, el primer modo:
nm = 1;
% El coeficiente depende de como esté apoyada la placa:
coef = 0.172;
% El módulo de Young es:
E1 = (0.99*(a^4)*(f1^2)*rho)/((coef^2)*(nm^4)*(e0^2));
E2 = (0.99*(a^4)*(f2^2)*rho)/((coef^2)*(nm^4)*(e0^2));
E3 = (0.99*(a^4)*(f3^2)*rho)/((coef^2)*(nm^4)*(e0^2));

disp(' ')
disp('Módulo de young: ')
E = [E1 E2 E3];
oE = isoutlier(E);
if(oE(1)==1)
    disp('Atención: El módulo de Young en la posición 1 es outlier')
else
    if(oE(2)==1)
        disp('Atención: El módulo de Young en la posición 2 es outlier')
    else
        if(oE(3)==1)
            disp('Atención: El módulo de Young en la posición 3 es outlier')
        end
    end
end
disp(['El módulo de Young en la posición 1 es: ', num2str(round((E1./1000000),2)), ' MPa'])
disp(['El módulo de Young en la posición 2 es: ', num2str(round((E2./1000000),2)), ' MPa'])
disp(['El módulo de Young en la posición 3 es: ', num2str(round((E3./1000000),2)), ' Mpa'])

% Cálculo de la rigidez a la flexión:
D1 = (2.*E1.*(e0/2)^3)./(3*(1-(0.1^2)));
D2 = (2.*E2.*(e0/2)^3)./(3*(1-(0.1^2)));
D3 = (2.*E3.*(e0/2)^3)./(3*(1-(0.1^2)));
disp(' ')
disp('Rigidez a la flexión: ')
D = [D1 D2 D3];
oD = isoutlier(D);
if(oD(1)==1)
    disp('Atención: La rigidez a flexión en la posición 1 es outlier')
else
    if(oD(2)==1)
        disp('Atención: La rigidez a flexión en la posición 2 es outlier')
    else
        if(oD(3)==1)
            disp('Atención: La rigidez a flexión en la posición 3 es outlier')
        end
    end
end
disp(['La rigidez a flexión en la posición 1 es: ', num2str(round(D1,1)), ' N⋅m'])
disp(['La rigidez a flexión en la posición 2 es: ', num2str(round(D2,1)), ' N⋅m'])
disp(['La rigidez a flexión en la posición 3 es: ', num2str(round(D3,1)), ' N⋅m'])